@@ -1,5 +1,6 @@
 # sql/dml.py
-# Copyright (C) 2009-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2009-2015 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,11 +10,13 @@ Provide :class:`.Insert`, :class:`.Update` and :class:`.Delete`.
 """
 
 from .base import Executable, _generative, _from_objects, DialectKWArgs
-from .elements import ClauseElement, _literal_as_text, Null, and_, _clone
+from .elements import ClauseElement, _literal_as_text, Null, and_, _clone, \
+    _column_as_key
 from .selectable import _interpret_as_from, _interpret_as_select, HasPrefixes
 from .. import util
 from .. import exc
 
+
 class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
     """Form the base for ``INSERT``, ``UPDATE``, and ``DELETE`` statements.
 
@@ -36,9 +39,8 @@ class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
             else:
                 return p
 
-        if isinstance(parameters, (list, tuple)) and \
-                parameters and \
-              isinstance(parameters[0], (list, tuple, dict)):
+        if (isinstance(parameters, (list, tuple)) and parameters and
+                isinstance(parameters[0], (list, tuple, dict))):
 
             if not self._supports_multi_parameters:
                 raise exc.InvalidRequestError(
@@ -82,7 +84,8 @@ class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
             stmt = table.update().\\
                       where(table.c.data == 'value').\\
                       values(status='X').\\
-                      returning(table.c.server_flag, table.c.updated_timestamp)
+                      returning(table.c.server_flag,
+                                table.c.updated_timestamp)
 
             for server_flag, updated_timestamp in connection.execute(stmt):
                 print(server_flag, updated_timestamp)
@@ -93,21 +96,20 @@ class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
         objects are typical, the elements can also be expressions::
 
             stmt = table.insert().returning(
-                      (table.c.first_name + " " + table.c.last_name).label('fullname')
-                    )
+                (table.c.first_name + " " + table.c.last_name).
+                label('fullname'))
 
         Upon compilation, a RETURNING clause, or database equivalent,
         will be rendered within the statement.   For INSERT and UPDATE,
         the values are the newly inserted/updated values.  For DELETE,
         the values are those of the rows which were deleted.
 
-        Upon execution, the values of the columns to be returned
-        are made available via the result set and can be iterated
-        using :meth:`.ResultProxy.fetchone` and similar.   For DBAPIs which do not
-        natively support returning values (i.e. cx_oracle),
-        SQLAlchemy will approximate this behavior at the result level
-        so that a reasonable amount of behavioral neutrality is
-        provided.
+        Upon execution, the values of the columns to be returned are made
+        available via the result set and can be iterated using
+        :meth:`.ResultProxy.fetchone` and similar.   For DBAPIs which do not
+        natively support returning values (i.e. cx_oracle), SQLAlchemy will
+        approximate this behavior at the result level so that a reasonable
+        amount of behavioral neutrality is provided.
 
         Note that not all databases/DBAPIs
         support RETURNING.   For those backends with no support,
@@ -128,7 +130,6 @@ class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
         """
         self._returning = cols
 
-
     @_generative
     def with_hint(self, text, selectable=None, dialect_name="*"):
         """Add a table hint for a single table to this
@@ -166,7 +167,7 @@ class UpdateBase(DialectKWArgs, HasPrefixes, Executable, ClauseElement):
             selectable = self.table
 
         self._hints = self._hints.union(
-                        {(selectable, dialect_name): text})
+            {(selectable, dialect_name): text})
 
 
 class ValuesBase(UpdateBase):
@@ -182,7 +183,7 @@ class ValuesBase(UpdateBase):
     def __init__(self, table, values, prefixes):
         self.table = _interpret_as_from(table)
         self.parameters, self._has_multi_parameters = \
-                            self._process_colparams(values)
+            self._process_colparams(values)
         if prefixes:
             self._setup_prefixes(prefixes)
 
@@ -193,9 +194,9 @@ class ValuesBase(UpdateBase):
 
         Note that the :class:`.Insert` and :class:`.Update` constructs support
         per-execution time formatting of the VALUES and/or SET clauses,
-        based on the arguments passed to :meth:`.Connection.execute`.  However,
-        the :meth:`.ValuesBase.values` method can be used to "fix" a particular
-        set of parameters into the statement.
+        based on the arguments passed to :meth:`.Connection.execute`.
+        However, the :meth:`.ValuesBase.values` method can be used to "fix" a
+        particular set of parameters into the statement.
 
         Multiple calls to :meth:`.ValuesBase.values` will produce a new
         construct, each one with the parameter list modified to include
@@ -228,8 +229,8 @@ class ValuesBase(UpdateBase):
 
          The :class:`.Insert` construct also supports multiply-rendered VALUES
          construct, for those backends which support this SQL syntax
-         (SQLite, Postgresql, MySQL).  This mode is indicated by passing a list
-         of one or more dictionaries/tuples::
+         (SQLite, Postgresql, MySQL).  This mode is indicated by passing a
+         list of one or more dictionaries/tuples::
 
             users.insert().values([
                                 {"name": "some name"},
@@ -247,9 +248,10 @@ class ValuesBase(UpdateBase):
          .. note::
 
              Passing a multiple values list is *not* the same
-             as passing a multiple values list to the :meth:`.Connection.execute`
-             method.  Passing a list of parameter sets to :meth:`.ValuesBase.values`
-             produces a construct of this form::
+             as passing a multiple values list to the
+             :meth:`.Connection.execute` method.  Passing a list of parameter
+             sets to :meth:`.ValuesBase.values` produces a construct of this
+             form::
 
                 INSERT INTO table (col1, col2, col3) VALUES
                                 (col1_0, col2_0, col3_0),
@@ -268,6 +270,19 @@ class ValuesBase(UpdateBase):
          .. versionadded:: 0.8
              Support for multiple-VALUES INSERT statements.
 
+        .. versionchanged:: 1.0.0 an INSERT that uses a multiple-VALUES
+           clause, even a list of length one,
+           implies that the :paramref:`.Insert.inline` flag is set to
+           True, indicating that the statement will not attempt to fetch
+           the "last inserted primary key" or other defaults.  The statement
+           deals with an arbitrary number of rows, so the
+           :attr:`.ResultProxy.inserted_primary_key` accessor does not apply.
+
+        .. versionchanged:: 1.0.0 A multiple-VALUES INSERT now supports
+           columns with Python side default values and callables in the
+           same way as that of an "executemany" style of invocation; the
+           callable is invoked for each row.   See :ref:`bug_3288`
+           for other details.
 
         .. seealso::
 
@@ -281,23 +296,23 @@ class ValuesBase(UpdateBase):
         """
         if self.select is not None:
             raise exc.InvalidRequestError(
-                        "This construct already inserts from a SELECT")
+                "This construct already inserts from a SELECT")
         if self._has_multi_parameters and kwargs:
             raise exc.InvalidRequestError(
-                        "This construct already has multiple parameter sets.")
+                "This construct already has multiple parameter sets.")
 
         if args:
             if len(args) > 1:
                 raise exc.ArgumentError(
-                            "Only a single dictionary/tuple or list of "
-                            "dictionaries/tuples is accepted positionally.")
+                    "Only a single dictionary/tuple or list of "
+                    "dictionaries/tuples is accepted positionally.")
             v = args[0]
         else:
             v = {}
 
         if self.parameters is None:
             self.parameters, self._has_multi_parameters = \
-                    self._process_colparams(v)
+                self._process_colparams(v)
         else:
             if self._has_multi_parameters:
                 self.parameters = list(self.parameters)
@@ -320,8 +335,8 @@ class ValuesBase(UpdateBase):
         if kwargs:
             if self._has_multi_parameters:
                 raise exc.ArgumentError(
-                            "Can't pass kwargs and multiple parameter sets "
-                            "simultaenously")
+                    "Can't pass kwargs and multiple parameter sets "
+                    "simultaenously")
             else:
                 self.parameters.update(kwargs)
 
@@ -339,47 +354,47 @@ class ValuesBase(UpdateBase):
             server_created_at = result.returned_defaults['created_at']
 
         When used against a backend that supports RETURNING, all column
-        values generated by SQL expression or server-side-default will be added
-        to any existing RETURNING clause, provided that
-        :meth:`.UpdateBase.returning` is not used simultaneously.   The column values
-        will then be available on the result using the
-        :attr:`.ResultProxy.returned_defaults` accessor as a
-        dictionary, referring to values keyed to the :class:`.Column` object
-        as well as its ``.key``.
+        values generated by SQL expression or server-side-default will be
+        added to any existing RETURNING clause, provided that
+        :meth:`.UpdateBase.returning` is not used simultaneously.  The column
+        values will then be available on the result using the
+        :attr:`.ResultProxy.returned_defaults` accessor as a dictionary,
+        referring to values keyed to the :class:`.Column` object as well as
+        its ``.key``.
 
         This method differs from :meth:`.UpdateBase.returning` in these ways:
 
         1. :meth:`.ValuesBase.return_defaults` is only intended for use with
            an INSERT or an UPDATE statement that matches exactly one row.
-           While the RETURNING construct in the general sense supports multiple
-           rows for a multi-row UPDATE or DELETE statement, or for special
-           cases of INSERT that return multiple rows (e.g. INSERT from SELECT,
-           multi-valued VALUES clause), :meth:`.ValuesBase.return_defaults`
-           is intended only
-           for an "ORM-style" single-row INSERT/UPDATE statement.  The row
-           returned by the statement is also consumed implcitly when
+           While the RETURNING construct in the general sense supports
+           multiple rows for a multi-row UPDATE or DELETE statement, or for
+           special cases of INSERT that return multiple rows (e.g. INSERT from
+           SELECT, multi-valued VALUES clause),
+           :meth:`.ValuesBase.return_defaults` is intended only for an
+           "ORM-style" single-row INSERT/UPDATE statement.  The row returned
+           by the statement is also consumed implcitly when
            :meth:`.ValuesBase.return_defaults` is used.  By contrast,
-           :meth:`.UpdateBase.returning` leaves the RETURNING result-set intact
-           with a collection of any number of rows.
+           :meth:`.UpdateBase.returning` leaves the RETURNING result-set
+           intact with a collection of any number of rows.
 
         2. It is compatible with the existing logic to fetch auto-generated
-           primary key values, also known as "implicit returning".  Backends that
-           support RETURNING will automatically make use of RETURNING in order
-           to fetch the value of newly generated primary keys; while the
+           primary key values, also known as "implicit returning".  Backends
+           that support RETURNING will automatically make use of RETURNING in
+           order to fetch the value of newly generated primary keys; while the
            :meth:`.UpdateBase.returning` method circumvents this behavior,
            :meth:`.ValuesBase.return_defaults` leaves it intact.
 
         3. It can be called against any backend.  Backends that don't support
            RETURNING will skip the usage of the feature, rather than raising
-           an exception.  The return value of :attr:`.ResultProxy.returned_defaults`
-           will be ``None``
+           an exception.  The return value of
+           :attr:`.ResultProxy.returned_defaults` will be ``None``
 
         :meth:`.ValuesBase.return_defaults` is used by the ORM to provide
         an efficient implementation for the ``eager_defaults`` feature of
         :func:`.mapper`.
 
         :param cols: optional list of column key names or :class:`.Column`
-         objects.  If omitted, all column expressions evaulated on the server
+         objects.  If omitted, all column expressions evaluated on the server
          are added to the returning list.
 
         .. versionadded:: 0.9.0
@@ -410,21 +425,22 @@ class Insert(ValuesBase):
     _supports_multi_parameters = True
 
     def __init__(self,
-                table,
-                values=None,
-                inline=False,
-                bind=None,
-                prefixes=None,
-                returning=None,
-                return_defaults=False,
-                **dialect_kw):
+                 table,
+                 values=None,
+                 inline=False,
+                 bind=None,
+                 prefixes=None,
+                 returning=None,
+                 return_defaults=False,
+                 **dialect_kw):
         """Construct an :class:`.Insert` object.
 
         Similar functionality is available via the
         :meth:`~.TableClause.insert` method on
         :class:`~.schema.Table`.
 
-        :param table: :class:`.TableClause` which is the subject of the insert.
+        :param table: :class:`.TableClause` which is the subject of the
+         insert.
 
         :param values: collection of values to be inserted; see
          :meth:`.Insert.values` for a description of allowed formats here.
@@ -432,15 +448,21 @@ class Insert(ValuesBase):
          dynamically render the VALUES clause at execution time based on
          the parameters passed to :meth:`.Connection.execute`.
 
-        :param inline: if True, SQL defaults will be compiled 'inline' into the
-          statement and not pre-executed.
+        :param inline: if True, no attempt will be made to retrieve the
+         SQL-generated default values to be provided within the statement;
+         in particular,
+         this allows SQL expressions to be rendered 'inline' within the
+         statement without the need to pre-execute them beforehand; for
+         backends that support "returning", this turns off the "implicit
+         returning" feature for the statement.
 
         If both `values` and compile-time bind parameters are present, the
         compile-time bind parameters override the information specified
         within `values` on a per-key basis.
 
-        The keys within `values` can be either :class:`~sqlalchemy.schema.Column`
-        objects or their string identifiers. Each key may reference one of:
+        The keys within `values` can be either
+        :class:`~sqlalchemy.schema.Column` objects or their string
+        identifiers. Each key may reference one of:
 
         * a literal data value (i.e. string, number, etc.);
         * a Column object;
@@ -460,6 +482,7 @@ class Insert(ValuesBase):
         ValuesBase.__init__(self, table, values, prefixes)
         self._bind = bind
         self.select = self.select_names = None
+        self.include_insert_from_select_defaults = False
         self.inline = inline
         self._returning = returning
         self._validate_dialect_kwargs(dialect_kw)
@@ -472,7 +495,7 @@ class Insert(ValuesBase):
             return ()
 
     @_generative
-    def from_select(self, names, select):
+    def from_select(self, names, select, include_defaults=True):
         """Return a new :class:`.Insert` construct which represents
         an ``INSERT...FROM SELECT`` statement.
 
@@ -491,36 +514,43 @@ class Insert(ValuesBase):
          is not checked before passing along to the database, the database
          would normally raise an exception if these column lists don't
          correspond.
+        :param include_defaults: if True, non-server default values and
+         SQL expressions as specified on :class:`.Column` objects
+         (as documented in :ref:`metadata_defaults_toplevel`) not
+         otherwise specified in the list of names will be rendered
+         into the INSERT and SELECT statements, so that these values are also
+         included in the data to be inserted.
 
-        .. note::
+         .. note:: A Python-side default that uses a Python callable function
+            will only be invoked **once** for the whole statement, and **not
+            per row**.
 
-           Depending on backend, it may be necessary for the :class:`.Insert`
-           statement to be constructed using the ``inline=True`` flag; this
-           flag will prevent the implicit usage of ``RETURNING`` when the
-           ``INSERT`` statement is rendered, which isn't supported on a backend
-           such as Oracle in conjunction with an ``INSERT..SELECT`` combination::
+         .. versionadded:: 1.0.0 - :meth:`.Insert.from_select` now renders
+            Python-side and SQL expression column defaults into the
+            SELECT statement for columns otherwise not included in the
+            list of column names.
 
-             sel = select([table1.c.a, table1.c.b]).where(table1.c.c > 5)
-             ins = table2.insert(inline=True).from_select(['a', 'b'], sel)
-
-        .. note::
-
-           A SELECT..INSERT construct in SQL has no VALUES clause.  Therefore
-           :class:`.Column` objects which utilize Python-side defaults
-           (e.g. as described at :ref:`metadata_defaults_toplevel`)
-           will **not** take effect when using :meth:`.Insert.from_select`.
+        .. versionchanged:: 1.0.0 an INSERT that uses FROM SELECT
+           implies that the :paramref:`.insert.inline` flag is set to
+           True, indicating that the statement will not attempt to fetch
+           the "last inserted primary key" or other defaults.  The statement
+           deals with an arbitrary number of rows, so the
+           :attr:`.ResultProxy.inserted_primary_key` accessor does not apply.
 
         .. versionadded:: 0.8.3
 
         """
         if self.parameters:
             raise exc.InvalidRequestError(
-                        "This construct already inserts value expressions")
+                "This construct already inserts value expressions")
 
         self.parameters, self._has_multi_parameters = \
-                self._process_colparams(dict((n, Null()) for n in names))
+            self._process_colparams(
+                dict((_column_as_key(n), Null()) for n in names))
 
         self.select_names = names
+        self.inline = True
+        self.include_insert_from_select_defaults = include_defaults
         self.select = _interpret_as_select(select)
 
     def _copy_internals(self, clone=_clone, **kw):
@@ -533,21 +563,22 @@ class Insert(ValuesBase):
 class Update(ValuesBase):
     """Represent an Update construct.
 
-    The :class:`.Update` object is created using the :func:`update()` function.
+    The :class:`.Update` object is created using the :func:`update()`
+    function.
 
     """
     __visit_name__ = 'update'
 
     def __init__(self,
-                table,
-                whereclause=None,
-                values=None,
-                inline=False,
-                bind=None,
-                prefixes=None,
-                returning=None,
-                return_defaults=False,
-                **dialect_kw):
+                 table,
+                 whereclause=None,
+                 values=None,
+                 inline=False,
+                 bind=None,
+                 prefixes=None,
+                 returning=None,
+                 return_defaults=False,
+                 **dialect_kw):
         """Construct an :class:`.Update` object.
 
         E.g.::
@@ -661,7 +692,6 @@ class Update(ValuesBase):
         self._validate_dialect_kwargs(dialect_kw)
         self._return_defaults = return_defaults
 
-
     def get_children(self, **kwargs):
         if self._whereclause is not None:
             return self._whereclause,
@@ -681,7 +711,7 @@ class Update(ValuesBase):
         """
         if self._whereclause is not None:
             self._whereclause = and_(self._whereclause,
-                    _literal_as_text(whereclause))
+                                     _literal_as_text(whereclause))
         else:
             self._whereclause = _literal_as_text(whereclause)
 
@@ -704,29 +734,30 @@ class Update(ValuesBase):
 class Delete(UpdateBase):
     """Represent a DELETE construct.
 
-    The :class:`.Delete` object is created using the :func:`delete()` function.
+    The :class:`.Delete` object is created using the :func:`delete()`
+    function.
 
     """
 
     __visit_name__ = 'delete'
 
     def __init__(self,
-            table,
-            whereclause=None,
-            bind=None,
-            returning=None,
-            prefixes=None,
-            **dialect_kw):
+                 table,
+                 whereclause=None,
+                 bind=None,
+                 returning=None,
+                 prefixes=None,
+                 **dialect_kw):
         """Construct :class:`.Delete` object.
 
         Similar functionality is available via the
         :meth:`~.TableClause.delete` method on
         :class:`~.schema.Table`.
 
-        :param table: The table to be updated.
+        :param table: The table to delete rows from.
 
         :param whereclause: A :class:`.ClauseElement` describing the ``WHERE``
-          condition of the ``UPDATE`` statement. Note that the
+          condition of the ``DELETE`` statement. Note that the
           :meth:`~Delete.where()` generative method may be used instead.
 
         .. seealso::
@@ -760,11 +791,10 @@ class Delete(UpdateBase):
 
         if self._whereclause is not None:
             self._whereclause = and_(self._whereclause,
-                    _literal_as_text(whereclause))
+                                     _literal_as_text(whereclause))
         else:
             self._whereclause = _literal_as_text(whereclause)
 
     def _copy_internals(self, clone=_clone, **kw):
         # TODO: coverage
         self._whereclause = clone(self._whereclause, **kw)
-
