@@ -1,5 +1,6 @@
 # testing/fixtures.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2015 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -17,6 +18,7 @@ from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 # whether or not we use unittest changes things dramatically,
 # as far as how py.test collection works.
 
+
 class TestBase(object):
     # A sequence of database names to always run, regardless of the
     # constraints below.
@@ -48,6 +50,7 @@ class TestBase(object):
         if hasattr(self, "tearDown"):
             self.tearDown()
 
+
 class TablesTest(TestBase):
 
     # 'once', None
@@ -88,20 +91,12 @@ class TablesTest(TestBase):
                 cls.run_create_tables = 'each'
             assert cls.run_inserts in ('each', None)
 
-        if cls.other is None:
-            cls.other = adict()
+        cls.other = adict()
+        cls.tables = adict()
 
-        if cls.tables is None:
-            cls.tables = adict()
-
-        if cls.bind is None:
-            setattr(cls, 'bind', cls.setup_bind())
-
-        if cls.metadata is None:
-            setattr(cls, 'metadata', sa.MetaData())
-
-        if cls.metadata.bind is None:
-            cls.metadata.bind = cls.bind
+        cls.bind = cls.setup_bind()
+        cls.metadata = sa.MetaData()
+        cls.metadata.bind = cls.bind
 
     @classmethod
     def _setup_once_inserts(cls):
@@ -197,9 +192,8 @@ class TablesTest(TestBase):
     def sql_count_(self, count, fn):
         self.assert_sql_count(self.bind, fn, count)
 
-    def sql_eq_(self, callable_, statements, with_sequences=None):
-        self.assert_sql(self.bind,
-                        callable_, statements, with_sequences)
+    def sql_eq_(self, callable_, statements):
+        self.assert_sql(self.bind, callable_, statements)
 
     @classmethod
     def _load_fixtures(cls):
@@ -221,6 +215,8 @@ class TablesTest(TestBase):
                  for column_values in rows[table]])
 
 from sqlalchemy import event
+
+
 class RemovesEvents(object):
     @util.memoized_property
     def _event_fns(self):
@@ -238,7 +234,6 @@ class RemovesEvents(object):
             super_.teardown()
 
 
-
 class _ORMTest(object):
 
     @classmethod
@@ -365,14 +360,14 @@ class DeclarativeMappedTest(MappedTest):
             def __init__(cls, classname, bases, dict_):
                 cls_registry[classname] = cls
                 return DeclarativeMeta.__init__(
-                        cls, classname, bases, dict_)
+                    cls, classname, bases, dict_)
 
         class DeclarativeBasic(object):
             __table_cls__ = schema.Table
 
         _DeclBase = declarative_base(metadata=cls.metadata,
-                            metaclass=FindFixtureDeclarative,
-                            cls=DeclarativeBasic)
+                                     metaclass=FindFixtureDeclarative,
+                                     cls=DeclarativeBasic)
         cls.DeclarativeBasic = _DeclBase
         fn()
 
