@@ -5,7 +5,7 @@ from sqlalchemy.testing.assertions import AssertsCompiledSQL, is_, \
 from sqlalchemy.testing import engines, fixtures
 from sqlalchemy import testing
 from sqlalchemy import Sequence, Table, Column, Integer, update, String,\
-    insert, func, MetaData, Enum, Index, and_, delete, select, cast
+    insert, func, MetaData, Enum, Index, and_, delete, select, cast, text
 from sqlalchemy.dialects.postgresql import ExcludeConstraint, array
 from sqlalchemy import exc, schema
 from sqlalchemy.dialects.postgresql import base as postgresql
@@ -166,6 +166,90 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             "VARCHAR(1), CHECK (somecolumn IN ('x', "
                             "'y', 'z')))")
 
+    def test_create_table_with_tablespace(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_tablespace='sometablespace')
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) TABLESPACE sometablespace")
+
+    def test_create_table_with_tablespace_quoted(self):
+        # testing quoting of tablespace name
+        m = MetaData()
+        tbl = Table(
+            'anothertable', m, Column("id", Integer),
+            postgresql_tablespace='table')
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            'CREATE TABLE anothertable (id INTEGER) TABLESPACE "table"')
+
+    def test_create_table_inherits(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_inherits='i1')
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) INHERITS ( i1 )")
+
+    def test_create_table_inherits_tuple(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_inherits=('i1', 'i2'))
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) INHERITS ( i1, i2 )")
+
+    def test_create_table_inherits_quoting(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_inherits=('Quote Me', 'quote Me Too'))
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            'CREATE TABLE atable (id INTEGER) INHERITS '
+            '( "Quote Me", "quote Me Too" )')
+
+    def test_create_table_with_oids(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_with_oids=True, )
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) WITH OIDS")
+
+        tbl2 = Table(
+            'anothertable', m, Column("id", Integer),
+            postgresql_with_oids=False)
+        self.assert_compile(
+            schema.CreateTable(tbl2),
+            "CREATE TABLE anothertable (id INTEGER) WITHOUT OIDS")
+
+    def test_create_table_with_oncommit_option(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_on_commit="drop")
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) ON COMMIT DROP")
+
+    def test_create_table_with_multiple_options(self):
+        m = MetaData()
+        tbl = Table(
+            'atable', m, Column("id", Integer),
+            postgresql_tablespace='sometablespace',
+            postgresql_with_oids=False,
+            postgresql_on_commit="preserve_rows")
+        self.assert_compile(
+            schema.CreateTable(tbl),
+            "CREATE TABLE atable (id INTEGER) WITHOUT OIDS "
+            "ON COMMIT PRESERVE ROWS TABLESPACE sometablespace")
+
     def test_create_partial_index(self):
         m = MetaData()
         tbl = Table('testtbl', m, Column('data', Integer))
@@ -212,6 +296,58 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                             '(data text_pattern_ops, data2 int4_ops)',
                             dialect=postgresql.dialect())
 
+    def test_create_index_with_text_or_composite(self):
+        m = MetaData()
+        tbl = Table('testtbl', m,
+                    Column('d1', String),
+                    Column('d2', Integer))
+
+        idx = Index('test_idx1', text('x'))
+        tbl.append_constraint(idx)
+
+        idx2 = Index('test_idx2', text('y'), tbl.c.d2)
+
+        idx3 = Index(
+            'test_idx2', tbl.c.d1, text('y'), tbl.c.d2,
+            postgresql_ops={'d1': 'x1', 'd2': 'x2'}
+        )
+
+        idx4 = Index(
+            'test_idx2', tbl.c.d1, tbl.c.d2 > 5, text('q'),
+            postgresql_ops={'d1': 'x1', 'd2': 'x2'}
+        )
+
+        idx5 = Index(
+            'test_idx2', tbl.c.d1, (tbl.c.d2 > 5).label('g'), text('q'),
+            postgresql_ops={'d1': 'x1', 'g': 'x2'}
+        )
+
+        self.assert_compile(
+            schema.CreateIndex(idx),
+            "CREATE INDEX test_idx1 ON testtbl (x)"
+        )
+        self.assert_compile(
+            schema.CreateIndex(idx2),
+            "CREATE INDEX test_idx2 ON testtbl (y, d2)"
+        )
+        self.assert_compile(
+            schema.CreateIndex(idx3),
+            "CREATE INDEX test_idx2 ON testtbl (d1 x1, y, d2 x2)"
+        )
+
+        # note that at the moment we do not expect the 'd2' op to
+        # pick up on the "d2 > 5" expression
+        self.assert_compile(
+            schema.CreateIndex(idx4),
+            "CREATE INDEX test_idx2 ON testtbl (d1 x1, (d2 > 5), q)"
+        )
+
+        # however it does work if we label!
+        self.assert_compile(
+            schema.CreateIndex(idx5),
+            "CREATE INDEX test_idx2 ON testtbl (d1 x1, (d2 > 5) x2, q)"
+        )
+
     def test_create_index_with_using(self):
         m = MetaData()
         tbl = Table('testtbl', m, Column('data', String))
@@ -253,6 +389,16 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "CREATE INDEX test_idx1 ON testtbl ((data + 5))"
         )
 
+    def test_create_index_concurrently(self):
+        m = MetaData()
+        tbl = Table('testtbl', m, Column('data', Integer))
+
+        idx1 = Index('test_idx1', tbl.c.data, postgresql_concurrently=True)
+        self.assert_compile(
+            schema.CreateIndex(idx1),
+            "CREATE INDEX CONCURRENTLY test_idx1 ON testtbl (data)"
+        )
+
     def test_exclude_constraint_min(self):
         m = MetaData()
         tbl = Table('testtbl', m,
